@@ -1,4 +1,5 @@
 from typing import Type
+from typing import TypeVar
 from typing import Callable
 from cfdata.tabular.processors.base import Processor
 
@@ -18,20 +19,26 @@ def register_initializer(name: str) -> Callable[[Callable], Callable]:
     return _register
 
 
-def register_processor(name: str) -> Callable[[Type], Type]:
-    return Processor.register(name)
+loss_type = Type[LossProtocol]
+metric_type = Type[MetricProtocol]
+PipelineType = TypeVar("PipelineType", bound="PipelineProtocol")
+processor_type = Type[Processor]
 
 
-def register_loss(name: str) -> Callable[[Type], Type]:
+def register_processor(name: str) -> Callable[[processor_type], processor_type]:
+    return Processor.register(name)  # type: ignore
+
+
+def register_loss(name: str) -> Callable[[loss_type], loss_type]:
     return LossProtocol.register(name)
 
 
-def register_metric(name: str) -> Callable[[Type], Type]:
+def register_metric(name: str) -> Callable[[metric_type], metric_type]:
     return MetricProtocol.register(name)
 
 
-def register_pipeline(name: str) -> Callable[[Type], Type]:
-    return PipelineProtocol.register(name)
+def register_pipeline(name: str) -> Callable[[PipelineType], PipelineType]:
+    return PipelineProtocol.register(name)  # type: ignore
 
 
 __all__ = [
