@@ -16,6 +16,7 @@ from typing import List
 from typing import Type
 from typing import Tuple
 from typing import Union
+from typing import TypeVar
 from typing import Callable
 from typing import Optional
 from typing import NamedTuple
@@ -46,10 +47,11 @@ from ..misc.toolkit import WithRegister
 
 
 data_modules: Dict[str, Type["DataModule"]] = {}
+DataModuleType = TypeVar("DataModuleType", bound="DataModule", covariant=True)
 
 
-class DataModule(WithRegister, metaclass=ABCMeta):
-    d: Dict[str, Type["DataModule"]] = data_modules
+class DataModule(WithRegister[DataModuleType], metaclass=ABCMeta):
+    d = data_modules  # type: ignore
 
     id_file = "id.txt"
     info_name = "info"
@@ -74,7 +76,7 @@ class DataModule(WithRegister, metaclass=ABCMeta):
         Saving.save_dict(self.info, self.info_name, folder)
 
     @classmethod
-    def _load_info(cls, folder: str) -> Dict[str, Any]:
+    def load_info(cls, folder: str) -> Dict[str, Any]:
         return Saving.load_dict(cls.info_name, folder)
 
     # api
@@ -91,10 +93,10 @@ class DataModule(WithRegister, metaclass=ABCMeta):
         folder = os.path.join(folder, cls.package_folder)
         with open(os.path.join(folder, cls.id_file), "r") as f:
             base = cls.get(f.read())
-        return base._load_info(folder)
+        return base.load_info(folder)
 
 
-@DataModule.register("dl")
+@DataModule.register("dl")  # type: ignore
 class DLDataModule(DataModule, metaclass=ABCMeta):
     test_transform: Optional[Callable]
     transform_file = "transform.pkl"
@@ -109,8 +111,8 @@ class DLDataModule(DataModule, metaclass=ABCMeta):
             dill.dump(self.test_transform, f)
 
     @classmethod
-    def _load_info(cls, folder: str) -> Dict[str, Any]:
-        info = super()._load_info(folder)
+    def load_info(cls, folder: str) -> Dict[str, Any]:
+        info = super().load_info(folder)
         transform_path = os.path.join(folder, cls.transform_file)
         if not os.path.isfile(transform_path):
             test_transform = None
@@ -121,10 +123,16 @@ class DLDataModule(DataModule, metaclass=ABCMeta):
         return info
 
 
-def get_weighted_indices(n: int, weights: Optional[np.ndarray]) -> np.ndarray:
+def get_weighted_indices(
+    n: int,
+    weights: Optional[np.ndarray],
+    ensure_all_occur: bool = False,
+) -> np.ndarray:
     indices = np.arange(n)
     if weights is not None:
         numbers = np.random.multinomial(n, weights)
+        if ensure_all_occur:
+            numbers += 1
         indices = indices.repeat(numbers)
     return indices
 
@@ -363,8 +371,8 @@ class CVLoader(DLLoader):
 cf_transforms: Dict[str, Type["Transforms"]] = {}
 
 
-class Transforms(WithRegister):
-    d: Dict[str, Type["Transforms"]] = cf_transforms
+class Transforms(WithRegister["Transforms"]):
+    d = cf_transforms
 
     fn: Any
 
@@ -446,7 +454,7 @@ class Compose(Transforms):
         return self.transform_list[0].need_numpy
 
 
-def _default_lmdb_path(folder: str, split: str) -> str:
+def default_lmdb_path(folder: str, split: str) -> str:
     return f"{folder}.{split}"
 
 
@@ -488,7 +496,7 @@ class ImageFolderDataset(Dataset):
             self.length = len(self.img_paths)
         else:
             self.lmdb_config = shallow_copy_dict(lmdb_config)
-            self.lmdb_config.setdefault("path", _default_lmdb_path(folder, split))
+            self.lmdb_config.setdefault("path", default_lmdb_path(folder, split))
             self.lmdb_config.setdefault("lock", False)
             self.lmdb_config.setdefault("meminit", False)
             self.lmdb_config.setdefault("readonly", True)
@@ -584,12 +592,16 @@ __all__ = [
     "MLDataset",
     "MLLoader",
     "DLDataset",
+    "LMDBItem",
     "CVDataset",
+    "ImageFolderDataset",
+    "InferenceImageFolderDataset",
     "DataLoader",
     "DLLoader",
     "CVLoader",
     "Transforms",
     "Function",
     "Compose",
+    "default_lmdb_path",
     "get_weighted_indices",
 ]
