@@ -28,7 +28,7 @@ from cftool.misc import Saving
 from torch.utils.data import Dataset
 from cfdata.tabular.api import TabularData
 
-from .core import _default_lmdb_path
+from .core import default_lmdb_path
 from .core import CVLoader
 from .core import DLLoader
 from .core import LMDBItem
@@ -314,8 +314,8 @@ class MLData(DLDataModule):
         Saving.save_dict(info, self.info_name, folder)
 
     @classmethod
-    def _load_info(cls, folder: str) -> Dict[str, Any]:
-        d = super()._load_info(folder)
+    def load_info(cls, folder: str) -> Dict[str, Any]:
+        d = super().load_info(folder)
         cf_data = d["cf_data"]
         if cf_data is None:
             return d
@@ -463,6 +463,27 @@ class ImageFolderData(CVDataModule):
         valid_loader = CVLoader(DataLoader(self.valid_data, **d), **kw)  # type: ignore
         return train_loader, valid_loader
 
+    @staticmethod
+    def switch_prefix(src: str, previous: str, now: str) -> None:
+        for split in ["train", "valid"]:
+            split_folder = os.path.join(src, split)
+            for file in os.listdir(split_folder):
+                if not file.endswith(".json"):
+                    continue
+                path = os.path.join(split_folder, file)
+                print(f"switching prefix for '{path}'")
+                with open(path, "r") as f:
+                    rs = json.load(f)
+                new_rs: Union[List, Dict[str, Any]]
+                if isinstance(rs, list):
+                    new_rs = [item.replace(previous, now) for item in rs]
+                else:
+                    new_rs = {}
+                    for k, v in rs.items():
+                        new_rs[k.replace(previous, now)] = v.replace(previous, now)
+                with open(path, "w", encoding="utf-8") as f:
+                    json.dump(new_rs, f, ensure_ascii=False)
+
 
 class InferenceImageFolderData(CVDataModule):
     def __init__(
@@ -495,24 +516,6 @@ class InferenceImageFolderData(CVDataModule):
         loader = self.dataset.make_loader(*args)
         return loader, None
 
-    @classmethod
-    def from_package(
-        cls,
-        src_folder: str,
-        *,
-        package_folder: str,
-        batch_size: int,
-        num_workers: int = 0,
-    ) -> "InferenceImageFolderData":
-        info = cls.load(package_folder)
-        transform = info["test_transform"]
-        return cls(
-            src_folder,
-            batch_size=batch_size,
-            num_workers=num_workers,
-            transform=transform,
-        )
-
 
 class PrepareResults(NamedTuple):
     data: ImageFolderData
@@ -694,11 +697,17 @@ def prepare_image_folder(
         labels_dict = {"": labels}
         label2idx = get_raw_2idx(sorted(set(labels)))
         dump_mappings = check_dump_mappings(label2idx)
+
+    open_file_from = lambda folder: lambda file: open(
+        os.path.join(folder, file), "w", encoding="utf-8"
+    )
+    open_tgt_file = open_file_from(tgt_folder)
+
     if dump_mappings:
-        with open(os.path.join(tgt_folder, f"{LABEL_KEY}2idx.json"), "w") as f:
-            json.dump(label2idx, f)
-        with open(os.path.join(tgt_folder, f"idx2{LABEL_KEY}.json"), "w") as f:
-            json.dump({v: k for k, v in label2idx.items()}, f)
+        with open_tgt_file(f"{LABEL_KEY}2idx.json") as f:
+            json.dump(label2idx, f, ensure_ascii=False)
+        with open_tgt_file(f"idx2{LABEL_KEY}.json") as f:
+            json.dump({v: k for k, v in label2idx.items()}, f, ensure_ascii=False)
 
     if extra_labels_dict is not None:
         for el_name, label_collection in extra_labels_dict.items():
@@ -714,10 +723,11 @@ def prepare_image_folder(
                 labels_dict[el_name] = [extra2idx[el] for el in label_collection]
                 dump_mappings = True
             if dump_mappings:
-                with open(os.path.join(tgt_folder, f"{el_name}2idx.json"), "w") as f:
-                    json.dump(extra2idx, f)
-                with open(os.path.join(tgt_folder, f"idx2{el_name}.json"), "w") as f:
-                    json.dump({v: k for k, v in extra2idx.items()}, f)
+                with open_tgt_file(f"{el_name}2idx.json") as f:
+                    json.dump(extra2idx, f, ensure_ascii=False)
+                with open_tgt_file(f"idx2{el_name}.json") as f:
+                    eld = {v: k for k, v in extra2idx.items()}
+                    json.dump(eld, f, ensure_ascii=False)
 
     # exclude samples
     if excluded_indices:
@@ -821,22 +831,23 @@ def prepare_image_folder(
                 ]
             )
         )
-        dtype_folder = os.path.join(tgt_folder, dtype)
-        with open(os.path.join(dtype_folder, "paths.json"), "w") as f_:
-            json.dump(new_paths, f_)
+        open_dtype_file = open_file_from(os.path.join(tgt_folder, dtype))
+
+        with open_dtype_file("paths.json") as f_:
+            json.dump(new_paths, f_, ensure_ascii=False)
         path_mapping = dict(zip(new_paths, valid_paths))
-        with open(os.path.join(dtype_folder, "path_mapping.json"), "w") as f_:
-            json.dump(path_mapping, f_)
+        with open_dtype_file("path_mapping.json") as f_:
+            json.dump(path_mapping, f_, ensure_ascii=False)
         for label_type, type_labels in merged_labels.items():
             delim = "_" if label_type else ""
             label_file = f"{label_type}{delim}{LABEL_KEY}.json"
-            with open(os.path.join(dtype_folder, label_file), "w") as f_:
-                json.dump(type_labels, f_)
+            with open_dtype_file(label_file) as f_:
+                json.dump(type_labels, f_, ensure_ascii=False)
         # lmdb
         if lmdb_config is None:
             return None
         local_lmdb_config = shallow_copy_dict(lmdb_config)
-        local_lmdb_config.setdefault("path", _default_lmdb_path(tgt_folder, dtype))
+        local_lmdb_config.setdefault("path", default_lmdb_path(tgt_folder, dtype))
         local_lmdb_config.setdefault("map_size", 1099511627776 * 2)
         db = lmdb.open(**local_lmdb_config)
         context = db.begin(write=True)
@@ -937,6 +948,7 @@ __all__ = [
     "DummyData",
     "MLData",
     "MLInferenceData",
+    "CVDataModule",
     "ImageFolderData",
     "InferenceImageFolderData",
     "DefaultPreparation",
