@@ -54,8 +54,8 @@ from .misc.internal_.trainer import make_trainer
 pipeline_dict: Dict[str, Type["PipelineProtocol"]] = {}
 
 
-class PipelineProtocol(WithRegister, metaclass=ABCMeta):
-    d: Dict[str, Type["PipelineProtocol"]] = pipeline_dict
+class PipelineProtocol(WithRegister["PipelineProtocol"], metaclass=ABCMeta):
+    d = pipeline_dict
 
     def __init__(self, *args: Any, **kwargs: Any):
         pass
@@ -96,7 +96,7 @@ class DLPipeline(PipelineProtocol, metaclass=ABCMeta):
     device_info: DeviceInfo
 
     pipeline_file: str = "pipeline.txt"
-    config_file: str = "config.json"
+    config_name: str = "config"
     trainer_config_file: str = "trainer_config.json"
     data_info_name: str = "data_info"
     metrics_log_file: str = "metrics.txt"
@@ -228,8 +228,7 @@ class DLPipeline(PipelineProtocol, metaclass=ABCMeta):
             self.trainer_config["data_info_name"] = self.data_info_name
             self.trainer_config["metrics_log_file"] = self.metrics_log_file
             self._write_pipeline_info(workplace)
-            with open(os.path.join(workplace, self.config_file), "w") as f:
-                json.dump(self.config, f)
+            Saving.save_dict(self.config, self.config_name, workplace)
 
     def _prepare_loss(self) -> None:
         if self.in_loading:
@@ -442,8 +441,7 @@ class DLPipeline(PipelineProtocol, metaclass=ABCMeta):
                 scores = json.load(rf)
             with open(os.path.join(pack_folder, SCORES_FILE), "w") as wf:
                 json.dump({new_file: scores[best_file]}, wf)
-            with open(os.path.join(workplace, cls.config_file), "r") as rf:
-                config = json.load(rf)
+            config = Saving.load_dict(cls.config_name, workplace)
             config_bundle = {
                 "config": config,
                 "device_info": DeviceInfo(cuda, None),
@@ -461,7 +459,7 @@ class DLPipeline(PipelineProtocol, metaclass=ABCMeta):
     @staticmethod
     def get_base(workplace: str) -> Type["DLPipeline"]:
         with open(os.path.join(workplace, DLPipeline.pipeline_file), "r") as f:
-            return DLPipeline.get(f.read())
+            return DLPipeline.get(f.read())  # type: ignore
 
     @staticmethod
     def load(
@@ -511,7 +509,7 @@ class DLPipeline(PipelineProtocol, metaclass=ABCMeta):
         onnx_file: str = "model.onnx",
         opset: int = 11,
         simplify: bool = True,
-        onnx_only: bool = False,
+        onnx_only: bool = True,
         forward_fn: Optional[Callable[[Dict[str, Any]], Dict[str, Any]]] = None,
         output_names: Optional[List[str]] = None,
         input_sample: Optional[tensor_dict_type] = None,
