@@ -20,6 +20,7 @@ from tqdm.autonotebook import tqdm
 from torch.optim import Optimizer
 from cftool.misc import update_dict
 from cftool.misc import shallow_copy_dict
+from cftool.misc import context_error_handler
 from torch.optim.lr_scheduler import _LRScheduler
 from torch.nn.parallel import DistributedDataParallel as DDP
 from torch.distributed.optim import ZeroRedundancyOptimizer as ZeRO
@@ -72,6 +73,22 @@ class OptimizerPack(NamedTuple):
     scheduler_config: Optional[Dict[str, Any]] = None
 
 
+class amp_context(context_error_handler):
+    def __init__(self, enabled: bool):
+        if not enabled:
+            self.ctx = None
+        else:
+            self.ctx = torch.cuda.amp.autocast(enabled=True)
+
+    def __enter__(self) -> None:
+        if self.ctx is not None:
+            self.ctx.__enter__()
+
+    def _normal_exit(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
+        if self.ctx is not None:
+            self.ctx.__exit__(exc_type, exc_val, exc_tb)
+
+
 class DefaultOptimizerSettings(NamedTuple):
     lr: float = 1.0e-3
     optimizer_name: str = "adam"
@@ -135,8 +152,8 @@ class DeviceInfo(NamedTuple):
         return torch.device("cpu" if self.cuda is None else f"cuda:{self.cuda}")
 
 
-class TrainerCallback(WithRegister):
-    d: Dict[str, Type["TrainerCallback"]] = callback_dict
+class TrainerCallback(WithRegister["TrainerCallback"]):
+    d = callback_dict
     is_rank_0: bool = True
 
     def __init__(self, *args: Any, **kwargs: Any):
@@ -430,12 +447,14 @@ class Trainer:
                 raise ValueError("Only `ConservativeMonitor` could be used in ddp mode")
         # ddp setup
         _setup_ddp()
+        torch.cuda.empty_cache()
+        torch.cuda.set_device(self.rank)
         if self.model_has_custom_steps and self.model.custom_ddp_initialization:
-            self.model.init_ddp(self)
+            self.model.init_ddp()
             return None
         if has_batch_norms(self.model):
             self.model = nn.SyncBatchNorm.convert_sync_batchnorm(self.model)
-        self.ddp_model = DDP(self.model.to(self.rank), device_ids=[self.rank])
+        self.ddp_model = DDP(self.model)
 
     def _init_finetune(self) -> None:
         if self.finetune_config is None:
@@ -483,7 +502,7 @@ class Trainer:
         # step
         step_default_cfg = {"step_size": 10 * self.state.num_step_per_epoch}
         # exponential
-        exp_gamma = (0.1 ** 0.1) ** (1.0 / self.state.num_step_per_epoch)
+        exp_gamma = (0.1**0.1) ** (1.0 / self.state.num_step_per_epoch)
         exp_default_cfg = {"gamma": exp_gamma}
         # cyclic
         cyclic_default_cfg = {
@@ -606,7 +625,29 @@ class Trainer:
 
     # core
 
-    def _clip_norm_step(self) -> None:
+    def post_loss_step(self, loss_dict: tensor_dict_type) -> None:
+        # backward
+        loss = loss_dict[LOSS_KEY]
+        self.grad_scaler.scale(loss).backward()
+        # clip norm
+        if self.clip_norm > 0.0:
+            self.clip_norm_step()
+        # optimize
+        self.optimizer_step()
+        self.scheduler_step()
+
+    def weighted_loss_score(self, loss_items: Dict[str, float]) -> float:
+        if not self.loss_metrics_weights:
+            return -loss_items[LOSS_KEY]
+        score = 0.0
+        for k, w in self.loss_metrics_weights.items():
+            v = loss_items.get(k)
+            if v is None:
+                continue
+            score -= v * w
+        return score
+
+    def clip_norm_step(self) -> None:
         for opt in self.optimizers.values():
             self.grad_scaler.unscale_(opt)
         self._gradient_norm = nn.utils.clip_grad_norm_(
@@ -614,13 +655,30 @@ class Trainer:
             max_norm=self.clip_norm,
         )
 
-    def _optimizer_step(self) -> None:
+    def optimizer_step(self) -> None:
         for opt in self.optimizers.values():
             self.grad_scaler.step(opt)
             self.grad_scaler.update()
         for param in self.model_for_training.parameters():
             param.grad = None
 
+    def scheduler_step(self) -> None:
+        lr_metric_logged = False
+        for key, scheduler in self.schedulers.items():
+            if scheduler is not None:
+                should_log_lr, kwargs = self._get_scheduler_settings(key, scheduler)
+                if should_log_lr:
+                    lr_metric_logged = True
+                    for callback in self.callbacks:
+                        callback.log_lr(
+                            f"lr-{key}",
+                            scheduler.get_last_lr()[0],
+                            self.state,
+                        )
+                scheduler.step(**shallow_copy_dict(kwargs))
+        if lr_metric_logged:
+            self.lr_metrics_updated = False
+
     def _get_scheduler_settings(
         self,
         key: str,
@@ -638,23 +696,6 @@ class Trainer:
             should_log_lr &= self.lr_metrics_updated
         return should_log_lr, kwargs
 
-    def _scheduler_step(self) -> None:
-        lr_metric_logged = False
-        for key, scheduler in self.schedulers.items():
-            if scheduler is not None:
-                should_log_lr, kwargs = self._get_scheduler_settings(key, scheduler)
-                if should_log_lr:
-                    lr_metric_logged = True
-                    for callback in self.callbacks:
-                        callback.log_lr(
-                            f"lr-{key}",
-                            scheduler.get_last_lr()[0],
-                            self.state,
-                        )
-                scheduler.step(**shallow_copy_dict(kwargs))
-        if lr_metric_logged:
-            self.lr_metrics_updated = False
-
     def _logging_step(self, metrics_outputs: MetricsOutputs) -> None:
         if not self.is_rank_0:
             return None
@@ -686,7 +727,7 @@ class Trainer:
                 self.intermediate = self.get_metrics(portion=self.valid_portion)
             else:
                 loss_dict = step_outputs.loss_dict
-                loss_score = self._weighted_loss_score(loss_dict)
+                loss_score = self.weighted_loss_score(loss_dict)
                 self.intermediate = MetricsOutputs(loss_score, loss_dict)
             self.lr_metrics_updated = True
             # logging
@@ -700,17 +741,6 @@ class Trainer:
                     terminate = True
         return MonitorResults(terminate, save_checkpoint, self.intermediate)
 
-    def _post_loss_step(self, loss_dict: tensor_dict_type) -> None:
-        # backward
-        loss = loss_dict[LOSS_KEY]
-        self.grad_scaler.scale(loss).backward()
-        # clip norm
-        if self.clip_norm > 0.0:
-            self._clip_norm_step()
-        # optimize
-        self._optimizer_step()
-        self._scheduler_step()
-
     def _step(self, batch_idx: int, batch: tensor_dict_type) -> StepOutputs:
         batch = to_device(batch, self.device)
         # kwargs
@@ -730,24 +760,13 @@ class Trainer:
                 loss_kwargs,
             )
         # forward & loss
-        with torch.cuda.amp.autocast(enabled=self.use_amp):
+        with amp_context(enabled=self.use_amp):
             forward_results = self.model(batch_idx, batch, self.state, **forward_kwargs)
             loss_dict = self.loss(forward_results, batch, self.state, **loss_kwargs)
         # post loss step
-        self._post_loss_step(loss_dict)
+        self.post_loss_step(loss_dict)
         return StepOutputs(forward_results, {k: v.item() for k, v in loss_dict.items()})
 
-    def _weighted_loss_score(self, loss_items: Dict[str, float]) -> float:
-        if not self.loss_metrics_weights:
-            return -loss_items[LOSS_KEY]
-        score = 0.0
-        for k, w in self.loss_metrics_weights.items():
-            v = loss_items.get(k)
-            if v is None:
-                continue
-            score -= v * w
-        return score
-
     # api
 
     @property
@@ -946,7 +965,7 @@ class Trainer:
         metric_outputs = outputs.metric_outputs
         if loss_items is not None:
             metrics.update(loss_items)
-            final_scores.append(self._weighted_loss_score(loss_items))
+            final_scores.append(self.weighted_loss_score(loss_items))
         if metric_outputs is not None:
             metrics.update(metric_outputs.metric_values)
             final_scores.append(metric_outputs.final_score)
@@ -1019,6 +1038,9 @@ class Trainer:
 __all__ = [
     "get_sorted_checkpoints",
     "Trainer",
+    "DeviceInfo",
+    "TqdmSettings",
+    "callback_dict",
     "TrainerCallback",
     "StepOutputs",
     "OptimizerPack",
